from rip import *
from utils import *

import argparse
import sys

# Note to self: Timeline assumes perfect care; as soon as it becomes sick, needs scolding,
# etc., they are taken care of, no care mistakes... etc.

stats = ripTamaStats()

def getMinutes(time):
  time = [int(x) for x in time.split(':')]
  time = time[0] * 60 + time[1]
  return time

def strip(val, toStrip):
  return int(val.replace(toStrip, ''))

def pad(x):
  if len(x) == 1:
    return '0' + x
  return x

def log(time, val):
  minutes = time % 60
  hours = int((time - minutes)/60)
  return '{}:{}: {}'.format(pad(str(hours)), pad(str(minutes)), val)

# FIXME! fragile for times around midnight
def isSleeping(time, sleepTime, wakeTime, isBaby):
  if isBaby:
    return time >= sleepTime and time < wakeTime
  else:
    return time >= sleepTime or time < wakeTime

def getInitialDiscipline(character, isTypeOneTeen):
  val = character['Initial Discipline']
  if val == '0%':
    if isTypeOneTeen:
      return 8
    else:
      return 0
  elif val == '25%':
    return 4
  elif val == '50%':
    return 8
  elif val == '75%':
    return 12
  elif val == '100%':
    return 15
  else:
    return -1
  
def handleDiscipline(disciplineCountdown, discipline, m214, disciplineMistakes, baseCountdown, currentTime, events):
  disciplineCountdown -= 1
  if disciplineCountdown == 0:
    disciplineCountdown = baseCountdown
    # (timeToDiscipline, m214) = shouldDiscipline(discipline, m214, events)
    (timeToDiscipline, m214) = shouldDiscipline(discipline, m214)
    if timeToDiscipline:
      events.append(log(currentTime, "Call for Discipline"))
      if disciplineMistakes == 0:
        discipline += 4
        if discipline == 16:
          discipline = 15
      else:
        disciplineMistakes -= 1

  return (disciplineCountdown, discipline, m214, disciplineMistakes)

def getTimeline(character, startTime, initialM214, initialAge, disciplineMistakes, suppressHearts, timeToNextHungryDrop, timeToNextHappyDrop, timeToNextPoop):
  isTypeOneTeen = (character[-1] == '+')
  if isTypeOneTeen:
    character = character[:-1]
  isNotAdult = character in ["Babytchi", "Marutchi", "Tamatchi", "Kuchitamatchi"]
  # lol
  isAdult = not isNotAdult
  isBaby = character == "Babytchi"
  hasReachedEvolutionTime = False

  character = stats[character]
  startTime = getMinutes(startTime)

  wakeTime = getMinutes(character['Wake Time'])
  sleepTime = getMinutes(character['Sleep Time'])
  if isBaby:
    sleepTime = (startTime + 40) % 1440
    wakeTime = (startTime + 45) % 1440

  sleeping = isSleeping(startTime, sleepTime, wakeTime, isBaby)

  timeToEnd = strip(character['Base Time to Evolution'], ' minutes')
  if isAdult:
    timeToEnd = strip(character['Base Time to Sickness'], ' minutes') * 3

  disciplineCountdown = strip(character['Countdown to Discipline'], ' hungry/happy heart decrements')
  baseCountdown = disciplineCountdown
  timeToEvolve = strip(character['Base Time to Evolution'], ' minutes')
  timeToSickness = strip(character['Base Time to Sickness'], ' minutes')
  timeToHungry = strip(character['Base Hungry Heart Loss Rate'], ' minutes')
  baseTimeToHungry = timeToHungry
  timeToPoop = 180
  if isBaby:
    timeToPoop = 15
  if timeToNextPoop is not None:
    timeToPoop = timeToNextPoop
  if timeToNextHungryDrop is not None:
    timeToHungry = timeToNextHungryDrop
  timeToHappy = strip(character['Base Happy Heart Loss Rate'], ' minutes')
  baseTimeToHappy = timeToHappy
  if isBaby:
    timeToHappy = 2
  if timeToNextHappyDrop is not None:
    timeToHappy = timeToNextHappyDrop
  # Pretty sure this starts as 0... but I also don't think it gets reset during evolution.
  # So it's a variable I want to be able to play around with.
  m214 = initialM214
  age = initialAge
  discipline = getInitialDiscipline(character, isTypeOneTeen)

  events = []

  currentTime = startTime
  while timeToEnd > 0:
    currentTime += 1
    if currentTime == 1440:
      currentTime = 0

    currentlySleeping = (sleeping == True)
    sleeping = isSleeping(currentTime, sleepTime, wakeTime, isBaby)
    if currentlySleeping and not sleeping:
      age += 1
      if hasReachedEvolutionTime:
        baseTimeToHungry = do_f3d(baseTimeToHungry)
        baseTimeToHappy = do_f3d(baseTimeToHappy)
      events.append(log(currentTime, "*** TAMA WAKES UP, AGE IS {} ***".format(age)))

    # I think technically, on the minute the Tama wakes up, the countdowns shouldn't decrement.
    # Although if they were already 0 when it woke up, they would cause whatever is the appropriate action.
    # I don't account for that here which will cause inaccuracies of a few minutes.

    if not sleeping:
      timeToEvolve -= 1
      if timeToEvolve == 0:
        events.append(log(currentTime, "Time to Evolution reached"))
        timeToEvolve = strip(character['Base Time to Evolution'], ' minutes')
        if isAdult:
          hasReachedEvolutionTime = True

      timeToSickness -= 1
      if timeToSickness == 0:
        events.append(log(currentTime, "Time to Sickness reached"))
        timeToSickness = strip(character['Base Time to Sickness'], ' minutes')
       
      timeToHungry -= 1  
      if timeToHungry == 0:
        if not isBaby:
          (disciplineCountdown, discipline, m214, disciplineMistakes) = handleDiscipline(disciplineCountdown, discipline, m214, disciplineMistakes, baseCountdown, currentTime, events)
        if not suppressHearts:
          events.append(log(currentTime, "Hungry Heart Decrement"))
        timeToHungry = baseTimeToHungry
        
      timeToHappy -= 1  
      if timeToHappy == 0:
        if not isBaby:
          (disciplineCountdown, discipline, m214, disciplineMistakes) = handleDiscipline(disciplineCountdown, discipline, m214, disciplineMistakes, baseCountdown, currentTime, events)
        if not suppressHearts:
          events.append(log(currentTime, "Happy Heart Decrement"))
        timeToHappy = baseTimeToHappy

      timeToPoop -= 1
      if timeToPoop == 0:
        if not suppressHearts:
          events.append(log(currentTime, "🚽"))
        timeToPoop = 180
        if isBaby:
          timeToPoop = 25
    
      timeToEnd -= 1

  events.append(log(currentTime, "End, m214 = {}, timeToHungry = {}, timeToHappy = {}, timeToPoop = {}".format(m214, timeToHungry, timeToHappy, timeToPoop)))
  return events

parser = argparse.ArgumentParser(description='Produces a timeline of events for the specified character')
parser.add_argument('character', help='The character whose timeline you want to view. Add + to Tamatchi or Kuchitamatchi to indicate a Type One teen.')
parser.add_argument('startTime', help='The time when this character evolved or hatched.')
parser.add_argument('--suppressHearts', '-s', action='store_true', help='Whether to suppress hungry/happy heart drops from the timeline output.')
parser.add_argument('--initialAge', '-i', type=int, help='Initial age for the Tama.', default=0, metavar='age')
parser.add_argument('--initialM214', '-m', type=int, help='Initial M[214] value for the Tama. (used when calculating discipline times)', default=0, metavar='m214')
parser.add_argument('--timeToNextHungryDrop', '-hu', help="Time to the next hungry heart drop. Defaults to the character's initial time to next drop.", type=int, metavar='minutes')
parser.add_argument('--timeToNextHappyDrop', '-ha', help="Time to the next happy heart drop. Defaults to the character's initial time to next drop.", type=int, metavar='minutes')
parser.add_argument('--timeToNextPoop', '-p', help="Time to the next poop. Defaults to 180 or 15 depending on whether character is a baby.", type=int, metavar='minutes')
parser.add_argument('--disciplineMistakes', '-d', help='Number of discipline mistakes you want to make. The first N discipline calls are assumed to be ignored, which influences when later calls are made.', default=0, type=int, metavar='mistakes')
args = parser.parse_args()

events = getTimeline(args.character, args.startTime, args.initialM214, args.initialAge, args.disciplineMistakes, args.suppressHearts, args.timeToNextHungryDrop, args.timeToNextHappyDrop, args.timeToNextPoop)
for e in events:
  print(e)
